''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    SIMULATION OF THE ACTION POTENTIAL OF THE SQUID GIANT AXON              '
'         AFTER HODGKIN & HUXLEY, J. PHYSIOL. 117:500 (1952).                '
'                                                                            '
'    Computer program based on Randall JE, Microcomputers                    '
'         and Physiological Simulation, Addison-Wesley,                      '
'         Reading, MA, pp. 157-198 (1980).                                   '
'                                                                            '
'    Latest revision by Stephen W. Jones, 2/25/89.                           '
'    Written for IBM PC-compatibles, using QuickBASIC 4.5.                   '
'    Requires CGA, EGA, Hercules, or AT&T graphics adapter.                  '
'                                                                            '
'    ACTION.BAS                                                              '
'                                                                            '
'    This version does current clamp.                                        '
'    It displays the membrane potential, and either the                      '
'         Na and K conductances, the open probabilities of the               '
'         n, m, and h gates, or the Na and K currents.                       '
'    Ion concentration ratios can be varied.                                 '
'    Na and K conductances can be reduced by "STX" and "TEA".                '
'    Na inactivation gates can be removed by "pronase".                      '
'                                                                            '
'    This program will return control to a menu program that calls           '
'         it via CHAIN.                                                      '
'    The menu program can also allow access to simplified versions, with     '
'         fewer options in a single on-screen menu.                          '
'                                                                            '
'    Data can be written to either .PRN (text, Lotus-compatible) or          '
'         .DAT (random access binary, CLAMPEX-compatible) files for          '
'         further analysis.                                                  '
'                                                                            '
'    Other modules needed:  EDITVAC, EDITVA, PARAMA                          '
'                                                                            '
'    Include files:                                                          '
'         DECLARES.INC (subprogram declarations and common blocks)           '
'         GRAPHICS.INC (subroutine Graphics and error traps)                 '
'         FILEOUT.INC (subroutines Dimensions, GetFileName,                  '
'              MakeClampexParameters, and MakeClampexLabels)                 '
'         PARAM.INC (subroutines for SUB NewParameters in module PARAMA)     '              
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    $INCLUDE: 'declares.inc'
 
		True = -1                     'allow logical if
		False = NOT True
				 
	CALL PeekInfo                      'get option, name of calling program
	IF Simulation% = 0 THEN            'If no option passed,
		CALL GetCommand                    'get option from command line.
	END IF
	GOSUB Dimensions                   'dimension arrays
	GOSUB Graphics                     'define colors, clear screen
	CALL PrintIntroText                'print introductory text
    
INITIAL:
   
	GOSUB Initializations              'initialize variables
	GOSUB MakeParameterArray           'use arrays for easier editing
  
NEXTPLOT:
  
	CALL NewParameters            'activate menus; allow exit from program
   
		IF ReInit THEN GOTO INITIAL
   
	GOSUB NewVariables            'convert back from arrays to variables
    
		IF ClearScreen OR ClearNext OR FirstScreen THEN CLS 0
		ON ERROR GOTO TRAP            'if error, stop run
    
	CALL Axes                     'draw and label x and y axes
	CALL Parameters               'display parameters on side of screen
	GOSUB Pharmacology            'STX, TEA, pronase, ion concentrations
	GOSUB FirstPoint              'calculate initial values
	GOSUB AbortRun                'print "hit any key to stop run"

ITERATE:                           'calculate another set of values
	    
		Abort$ = INKEY$
		IF Abort$ = Esc$ THEN
			IF FileOut <> 1 THEN CLOSE #1 'Close output file, if any
			GOTO NEXTPLOT       'stop run; what next?
		END IF

	GOSUB Rates                   'rate constants
	GOSUB Gates                   'n, m, and h gates
	GOSUB Currents                'conductances and currents
	GOSUB Stimulus                'add stim current if any
	GOSUB Voltage                 'calculate next voltage
	GOSUB Plots                   'graph new currents, voltage

		T& = T& + 1                        'increment point #
		IF T& < Tint GOTO ITERATE          'next point or next plot
		IF Measure THEN GOSUB WriteMeasure
		IF FileOut <> 1 THEN CLOSE #1      'Close output file, if any

GOTO NEXTPLOT                      'next run


'  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '  '

TRAP:
	ErrorType = ERR                         'Main error trap
	IF ErrorType = 6 OR ErrorType = 5 THEN
		SOUND 500, 1                            'Sound a warning
		RESUME NEXTPLOT                         'Error 6 = overflow
	END IF
	ON ERROR GOTO 0

'    $INCLUDE: 'graphics.inc'

'    $INCLUDE: 'fileout.inc'

Initializations:                   'Initialize variables
	ReInit = False
	NextMenu = 0
	FileNumber = 0                     'for naming output files
	SpecialChar$ = "i"                      '"i" for I-clamp
    
	SidebarCol = 69                    'Starting point for sidebar menus
	PrintDefaultAt = 14                'Column for printing current values
   
	PlotType = 1                  'Display menu parameters
	LowerPlot = 1
	TimeScale = 4                           '10 msec timebase
	Time = 10                               '10 msec timebase
	ClearNext = False
	ClearScreen = False
	FirstScreen = True
   
	RP = -60                           'Stimuli menu parameters
	uA = 100
	Duration = .1
	Start = 0
	uA2 = 0
	Duration2 = 0
	Start2 = 0

	STX = 0                            'Pharmacology menu parameters
	TEA = 0
	IntactH = 100
	EKModel = -72                      'Equilibrium potentials on H-H model,
	ENaModel = 55                           'assuming rp = -60
	NaRatio = 1
	KRatio = 1
	SlowCalc = False

	FileOut = 1                        'Output menu parameter
    
	gKMax = 36                         'Potassium channel kinetics parameters
	nExp = 4
	Can = .01
	Van0 = -10
	Van = 10
	anExp = False
	Cbn = .125
	Vbn0 = 0
	Vbn = -80

	gNaMax = 120                       'Sodium activation parameters
	mExp = 3
	Cam = .1
	Vam0 = -25
	Vam = 10
	amExp = False
	Cbm = 4
	Vbm0 = 0
	Vbm = -18

	hExp = 1                           'Sodium inactivation parameters
	Cah = .07
	Vah0 = 0
	Vah = -20
	Cbh = 1
	Vbh0 = -30
	Vbh = 10
	bhExp = False
	gLeak = .3

	RETURN

MakeParameterArray:                     'editing routine uses this array
	Parameter(2, 1) = PlotType
	Parameter(2, 2) = LowerPlot
	Parameter(2, 3) = TimeScale
	Parameter(2, 4) = ClearScreen
	Parameter(2, 5) = False
   
	Parameter(3, 1) = RP
	Parameter(3, 2) = uA
	Parameter(3, 3) = Start
	Parameter(3, 4) = Duration
	Parameter(3, 5) = uA2
	Parameter(3, 6) = Start2
	Parameter(3, 7) = Duration2

	Parameter(4, 1) = STX
	Parameter(4, 2) = TEA
	Parameter(4, 3) = IntactH
	Parameter(4, 4) = NaRatio
	Parameter(4, 5) = KRatio
	Parameter(4, 6) = SlowCalc

	Parameter(5, 1) = RP
	Parameter(5, 2) = uA
	Parameter(5, 3) = Start
	Parameter(5, 4) = Duration
	Parameter(5, 5) = uA2
	Parameter(5, 6) = Start2
	Parameter(5, 7) = Duration2
	Parameter(5, 8) = True
	Parameter(5, 9) = False
  
	Parameter(6, 1) = RP
	Parameter(6, 2) = uA
	Parameter(6, 3) = Start
	Parameter(6, 4) = Duration
	Parameter(6, 5) = uA2
	Parameter(6, 6) = Start2
	Parameter(6, 7) = Duration2
	Parameter(6, 8) = False
	Parameter(6, 9) = False
   
	Parameter(7, 1) = RP
	Parameter(7, 2) = 300
	Parameter(7, 3) = Start
	Parameter(7, 4) = Duration
	Parameter(7, 5) = STX
	Parameter(7, 6) = TEA
	Parameter(7, 7) = IntactH
	Parameter(7, 8) = False
	Parameter(7, 9) = False
 
	Parameter(8, 1) = RP
	Parameter(8, 2) = 300
	Parameter(8, 3) = Start
	Parameter(8, 4) = Duration
	Parameter(8, 5) = NaRatio
	Parameter(8, 6) = KRatio
	Parameter(8, 7) = False
	Parameter(8, 8) = False
 
	Parameter(9, 1) = FileOut

	Parameter(10, 1) = StartM(1)
	Parameter(10, 2) = EndM(1)
	Parameter(10, 3) = StartM(2)
	Parameter(10, 4) = EndM(2)
	Parameter(10, 5) = StartM(3)
	Parameter(10, 6) = EndM(3)
	Parameter(10, 7) = StartM(4)
	Parameter(10, 8) = EndM(4)

	Parameter(11, 1) = gKMax
	Parameter(11, 2) = nExp
	Parameter(11, 3) = Can
	Parameter(11, 4) = -60 - Van0            'Change voltages to HH units
	Parameter(11, 5) = Van
	Parameter(11, 6) = anExp
	Parameter(11, 7) = Cbn
	Parameter(11, 8) = -60 - Vbn0
	Parameter(11, 9) = Vbn

	Parameter(12, 1) = gNaMax
	Parameter(12, 2) = mExp
	Parameter(12, 3) = Cam
	Parameter(12, 4) = -60 - Vam0
	Parameter(12, 5) = Vam
	Parameter(12, 6) = amExp
	Parameter(12, 7) = Cbm
	Parameter(12, 8) = -60 - Vbm0
	Parameter(12, 9) = Vbm

	Parameter(13, 1) = hExp
	Parameter(13, 2) = Cah
	Parameter(13, 3) = -60 - Vah0
	Parameter(13, 4) = Vah
	Parameter(13, 5) = Cbh
	Parameter(13, 6) = -60 - Vbh0
	Parameter(13, 7) = Vbh
	Parameter(13, 8) = bhExp
	Parameter(13, 9) = gLeak

	RETURN
    
NewVariables:                           'Convert to individual variables
	PlotType = Parameter(2, 1)              'Display menu parameters
	LowerPlot = Parameter(2, 2)
	TimeScale = Parameter(2, 3)
	ClearScreen = Parameter(2, 4)
	ClearNext = Parameter(2, 5)
  
	RP = Parameter(3, 1)                    'Stimuli menu parameters
	uA = Parameter(3, 2)
	Start = Parameter(3, 3)
	Duration = Parameter(3, 4)
	uA2 = Parameter(3, 5)
	Start2 = Parameter(3, 6)
	Duration2 = Parameter(3, 7)
    
	STX = Parameter(4, 1)                   'Pharmacology menu parameters
	TEA = Parameter(4, 2)
	IntactH = Parameter(4, 3)
	NaRatio = Parameter(4, 4)
	KRatio = Parameter(4, 5)
	SlowCalc = Parameter(4, 6)
    
	FileOut = Parameter(9, 1)               'Output menu parameter
    
	StartM(1) = Parameter(10, 1)            'Measurement menu parameters
	EndM(1) = Parameter(10, 2)
	StartM(2) = Parameter(10, 3)
	EndM(2) = Parameter(10, 4)
	StartM(3) = Parameter(10, 5)
	EndM(3) = Parameter(10, 6)
	StartM(4) = Parameter(10, 7)
	EndM(4) = Parameter(10, 8)
	Measure = False
	FOR i = 1 TO 8
		IF Parameter(10, i) > 0 THEN Measure = True
	NEXT i

	gKMax = Parameter(11, 1)                'Potassium kinetics parameters
	nExp = Parameter(11, 2)
	Can = Parameter(11, 3)
	Van0 = -60 - Parameter(11, 4)
	Van = Parameter(11, 5)
	anExp = Parameter(11, 6)
	Cbn = Parameter(11, 7)
	Vbn0 = -60 - Parameter(11, 8)
	Vbn = Parameter(11, 9)

	gNaMax = Parameter(12, 1)               'Sodium activation parameters
	mExp = Parameter(12, 2)
	Cam = Parameter(12, 3)
	Vam0 = -60 - Parameter(12, 4)
	Vam = Parameter(12, 5)
	amExp = Parameter(12, 6)
	Cbm = Parameter(12, 7)
	Vbm0 = -60 - Parameter(12, 8)
	Vbm = Parameter(12, 9)

	hExp = Parameter(13, 1)                 'Sodium inactivation parameters
	Cah = Parameter(13, 2)
	Vah0 = -60 - Parameter(13, 3)
	Vah = Parameter(13, 4)
	Cbh = Parameter(13, 5)
	Vbh0 = -60 - Parameter(13, 6)
	Vbh = Parameter(13, 7)
	bhExp = Parameter(13, 8)
	gLeak = Parameter(13, 9)

	SELECT CASE Simulation%            'For simplified versions:
		CASE 5
			RP = Parameter(5, 1)          'Action potential menu parameters
			uA = Parameter(5, 2)
			Start = Parameter(5, 3)
			Duration = Parameter(5, 4)
			uA2 = Parameter(5, 5)
			Start2 = Parameter(5, 6)
			Duration2 = Parameter(5, 7)
			IF Parameter(5, 8) THEN
				LowerPlot = 1
			ELSE LowerPlot = 2
			END IF
			ClearNext = Parameter(5, 9)
		CASE 6
			RP = Parameter(6, 1)          'Expanded scales menu parameters
			uA = Parameter(6, 2)
			Start = Parameter(6, 3)
			Duration = Parameter(6, 4)
			uA2 = Parameter(6, 5)
			Start2 = Parameter(6, 6)
			Duration2 = Parameter(6, 7)
			ClearNext = Parameter(6, 8)
			ClearScreen = Parameter(6, 9)
			LowerPlot = 3
			PlotType = 2
		CASE 7
			RP = Parameter(7, 1)          'Drug menu parameters
			uA = Parameter(7, 2)
			Start = Parameter(7, 3)
			Duration = Parameter(7, 4)
			STX = Parameter(7, 5)
			TEA = Parameter(7, 6)
			IntactH = Parameter(7, 7)
			IF IntactH < 100 THEN
				SlowCalc = True
			ELSE SlowCalc = False
			END IF
			IF Parameter(7, 8) THEN
				TimeScale = 6
			ELSE TimeScale = 4
			END IF
			ClearNext = Parameter(7, 9)
		CASE 8
			RP = Parameter(8, 1)          'Ionic menu parameters
			uA = Parameter(8, 2)
			Start = Parameter(8, 3)
			Duration = Parameter(8, 4)
			NaRatio = Parameter(8, 5)
			KRatio = Parameter(8, 6)
			IF Parameter(8, 7) THEN
				TimeScale = 6
			ELSE TimeScale = 4
			END IF
			ClearNext = Parameter(8, 8)
		CASE ELSE
	END SELECT
    
	Time = TimeOption(TimeScale)            'Duration of a sweep, and
	Tint = IntervalOption(TimeScale)             'calculation interval.
	IF SlowCalc THEN
		Tint = 100 * Time                  'Extra points if requested
	END IF
	dt = Time / Tint
    
	RETURN

Pharmacology:
	KdStx = 1                                    'dissoc constant = 1 nM
	KdTea = 1                                    'dissoc constant = 1 mM
	UnblockedNa = KdStx / (STX + KdStx)               'STX
	UnblockedK = KdTea / (TEA + KdTea)                'TEA

	RETURN

FirstPoint:
	T& = 0                                  'T& is the iteration number
	T = T& / Tint                           'this helps avoid overflow
	TPlot% = 1 + XStart + T * 500 / XScale  'used for plotting
	mV = RP                                 'initial voltage = desired r. p.
	StimOn = False
	FirstScreen = False

	Max(1) = 0                              'initialize measurements
	Max(2) = 0                                   '1 and 2 absolute value
	Max(3) = -10 ^ 6                             'detects maximum
	Max(4) = 10 ^ 6                              'detects minimum

	gKleak = .82205 * gLeak                 'Chosen to give Eleak = -49.4,
	gNaleak = .17795 * gLeak                     'if leak conductance = 0.3

	EK = EKModel + 25 * LOG(KRatio)         'RT/nF = 25 mV, natural log
	ENa = ENaModel + 25 * LOG(NaRatio)
	EKHH = -EK - 60                         'convert to H-H units
	ENaHH = -ENa - 60

	GOSUB Rates                        'get rate constants

	n = an / (an + bn)                      'steady-state values at initial V
	m = am / (am + bm)
	h = ah / (ah + bh)

	GOSUB Currents                     'calculate currents

	SELECT CASE FileOut                'Write data to files, if requested
		CASE 2                                       '.PRN file (Lotus)
			FileExt$ = ".prn"
			GOSUB GetFileName
			OPEN FileName$ FOR OUTPUT AS #1
			GOSUB MakeLotusHeader
		CASE 3                                       '.DAT file (pCLAMP)
			FileExt$ = ".dat"
			GOSUB GetFileName
		    
			OutputGain = 50                              'gain (mV/mV)
			GOSUB MakeClampexParameters                  'Construct header
		    
			Comment$ = "Output from squid axon model"    'Comment
			Label$ = "              mV"
			GOSUB MakeClampexLabels                      'Write labels
		    
			OPEN FileName$ FOR RANDOM AS #1 LEN = 2      'Data as integers,
			StartPut = 512                                    'start here.
		CASE ELSE
	END SELECT

	GOSUB Voltage                 'calculate initial voltage (no stimulus)
	GOSUB Plots                   'graph new currents, voltage

	T& = T& + 1                   'increment point number
	
	RETURN

MakeLotusHeader:
	SELECT CASE LowerPlot
		CASE 3                                                 'currents
			WRITE #1, "TIME", "Voltage", "I(Na)", "I(K)"
			WRITE #1, "ms", "mV", "mA/cm2", "mA/cm2"
		CASE 2                                                 'gates
			WRITE #1, "TIME", "Voltage", "Na (m)", "Na (h)", "K (n)"
			WRITE #1, "ms", "mV", "gate", "gate", "gate"
		CASE ELSE                                              'conductances
			WRITE #1, "TIME", "Voltage", "G(Na)", "G(K)"
			WRITE #1, "ms", "mV", "mS/cm2", "mS/cm2"
	END SELECT
  
	RETURN

AbortRun:
	IF Ega THEN COLOR Red                   'print in red
	LOCATE 20, SidebarCol
	PRINT " HIT <Esc>  "
	LOCATE , SidebarCol
	PRINT "TO STOP RUN "                    'main program checks for input

	RETURN

Rates:
	V = -mV - 60                       'set rp = -60 mV.  V is in H-H units
	IF V - Van0 = 0 THEN V = V - .01         'avoid division by zero
	IF V - Vam0 = 0 THEN V = V - .01

	IF anExp THEN
		an = Can * EXP(-(V - Van0) / Van)
	ELSE an = Can * (V - Van0) / (EXP((V - Van0) / Van) - 1)    'a=opening
	END IF
	IF amExp THEN
		am = Cam * EXP(-(V - Vam0) / Vam)
	ELSE am = Cam * (V - Vam0) / (EXP((V - Vam0) / Vam) - 1)
	END IF
	ah = Cah * EXP(-(V - Vah0) / Vah)
   
	bn = Cbn * EXP(-(V - Vbn0) / Vbn)                           'b=closing
	bm = Cbm * EXP(-(V - Vbm0) / Vbm)
	IF bhExp THEN
		bh = Cbh * EXP(-(V - Vbh0) / Vbh)
	ELSE bh = Cbh / (EXP((V - Vbh0) / Vbh) + 1)
	END IF

	RETURN

Gates:
	dn = an * (1 - n) - bn * n         'increments
	dm = am * (1 - m) - bm * m
	dh = ah * (1 - h) - bh * h

	n = n + dn * dt                    'current values of gate p(open)
	m = m + dm * dt
	h = h + dh * dt

	IF n > 1 THEN n = 1                'recover from extrapolation errors
	IF m > 1 THEN m = 1
	IF h > 1 THEN h = 1
	IF n < 0 THEN n = 0
	IF m < 0 THEN m = 0
	IF h < 0 THEN h = 0

	RETURN

Currents:
	NetH = ((IntactH * h + (100 - IntactH)) / 100) ^ hExp  'pronase
	gK = UnblockedK * gKMax * n ^ nExp                     'n to 4th
	gNa = UnblockedNa * gNaMax * m ^ mExp * NetH           'm cubed h

	iK = gK * (V - EKHH)                              'calculate currents
	iNa = gNa * (V - ENaHH)
	IL = gKleak * (V - EKHH) + gNaleak * (V - ENaHH)  'parallel batteries
	IT = iK + iNa + IL

	RETURN

Stimulus:
	OldStim = StimOn
	OnOrOff = False
	StimOn = False

	IF T& > Start / dt + .1 AND T& < .1 + (Start + Duration) / dt THEN
		IT = IT + uA
		StimOn = True
	END IF
	IF T& > Start2 / dt + .1 AND T& < .1 + (Start2 + Duration2) / dt THEN
		IT = IT + uA2
		StimOn = True
	END IF
	IF OldStim <> StimOn THEN OnOrOff = True     'checks for stim on or off

	RETURN

Voltage:
	dV = IT * dt
	V = V - dV                                   'calculate new voltage
	mV = -V - 60                                 'convert to usual convention

	RETURN

Plots:
	PlotDot = False
	TLast% = TPlot%
	T = T& / Tint                                'this helps avoid overflow
	ms = T * Time                                'time in ms
	TPlot% = 1 + XStart + T * 500 / XScale       'convert time to screen units
	IF T& > 0 AND T& <= 50 THEN
		PlotDot = True
		TDot = XStart + T& * 10 / XScale        'used for dotted lines
	END IF

	Twice = False
	IF TPlot% > TLast% + 1 THEN             'Avoid blank spaces horizontally
		Twice = True                            'with Hercules adapter
		TPlot% = TPlot% - 1                     'when 500 points plotted.
	END IF
plotPointsTwice:
	IF Ega THEN COLOR Yellow
	SELECT CASE PlotType                         'Plot voltage:
		CASE 2
			GOSUB PlotVoltageExpanded               'expanded scale, or
		CASE ELSE
			GOSUB PlotVoltage                       'normal scale.
	END SELECT

	SELECT CASE LowerPlot                        'Plot:
		CASE 3
			GOSUB PlotCurrents                      'currents,
		CASE 2
			GOSUB PlotGates                         'gates, or
		CASE ELSE
			GOSUB PlotConductances                  'conductances.
	END SELECT
    
	IF StimOn THEN PSET (TPlot%, 285 / YScale)   'plot stim 285-300
	IF OnOrOff THEN                              'if stim on/off now
		LINE (TPlot%, 285 / YScale)-(TPlot%, 300 / YScale)
	END IF
    
	IF Twice THEN                                'Plot some points twice
		TPlot% = TPlot% + 1                          'if Hercules card.
		Twice = False
		GOTO plotPointsTwice
	END IF
    
	SELECT CASE FileOut                     'Write to output file (if any)
		CASE 2
			GOSUB WriteToLotus
		CASE 3
			GOSUB WriteToPClamp
		CASE ELSE
	END SELECT

	IF Measure THEN GOSUB Measurements

	RETURN

WriteToLotus:
	ms = T * Time                                'time, in ms
	SELECT CASE LowerPlot
		CASE 3
			WRITE #1, ms, mV, iNa, iK          '.PRN file for Lotus
		CASE 2
			WRITE #1, ms, mV, m, h, n
		CASE ELSE
			WRITE #1, ms, mV, gNa, gK
	END SELECT

	RETURN
    
WriteToPClamp:
	ScaledCurrent = mV * 2048 * OutputGain / 10000
	IF ScaledCurrent > 2048 THEN ScaledCurrent = 2048
	IF ScaledCurrent < -2048 THEN ScaledCurrent = -2048
	ClampexData = CINT(ScaledCurrent)
	StartPut = StartPut + 1
	IF T& < PointsToWrite THEN
		PUT #1, StartPut, ClampexData           '.DAT file for pCLAMP
	END IF
	IF T& = 0 THEN
		FOR i = 1 TO FirstPoints
			StartPut = StartPut + 1
			PUT #1, StartPut, ClampexData
		NEXT i
	END IF

	RETURN

PlotVoltage:
	PSET (TPlot%, (90 - 1.4 * mV) / YScale)      '0 mV at 90; 1.4 points/mV
	IF Ega THEN COLOR White
	IF PlotDot THEN
		PSET (TDot, 90 / YScale)                'dotted line at V = 0
		PSET (TDot, (90 - 1.4 * RP) / YScale)   'dotted line V = HP
	END IF

	RETURN

PlotVoltageExpanded:
	PSET (TPlot%, (-120 - 4.2 * mV) / YScale)    '0 mV at -120; 4.2 points/mV
	IF Ega THEN COLOR White
	IF PlotDot THEN PSET (TDot, (-120 - 4.2 * RP) / YScale)     'dots, V = HP

	RETURN
    
PlotCurrents:
	IF Ega THEN COLOR Blue
	IF iNa < 36 THEN
		PSET (TPlot%, (272 - 2.1 * iNa) / YScale)    'I Na, blue
	END IF
	IF Ega THEN COLOR Green
	IF iK > -36 AND (HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2)) THEN
		PSET (TPlot%, (272 + 2.1 * iK) / YScale)     'I K, green
	END IF
	IF Ega THEN COLOR White                           'I = 0 at 272
	IF PlotDot THEN PSET (TDot, 272 / YScale)         'dotted line at I = 0

	RETURN

PlotGates:
	IF Ega THEN COLOR Blue
	PSET (TPlot%, (272 - 63 * m) / YScale)            'Na "m" gate in blue
	IF Ega THEN COLOR Green
	IF HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2) THEN
		PSET (TPlot%, (272 - 63 * n) / YScale)       'K "n" gate in green
	END IF                                            'dotted line if mono
	IF Ega THEN COLOR Red
	IF HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2) THEN
		PSET (TPlot%, (272 - 63 * NetH) / YScale)    'Na "h" gate in red
	END IF
	IF Ega THEN COLOR White
	IF PlotDot THEN
		PSET (TDot, 209 / YScale)                    '1 => 209; 0 => 272
		PSET (TDot, 272 / YScale)                         'dotted lines
	END IF

	RETURN

PlotConductances:
	IF Ega THEN COLOR Blue
	PSET (TPlot%, (272 - 21 * gNa / 20) / YScale)     'G Na, blue
	IF Ega THEN COLOR Green
	IF HiRes = 9 OR TPlot% / 2 = INT(TPlot% / 2) THEN
		PSET (TPlot%, (272 - 21 * gK / 20) / YScale) 'G K, green
	END IF
	IF Ega THEN COLOR White                           'G = 0 at 272
	IF PlotDot THEN PSET (TDot, 272 / YScale)         'dotted line at G = 0

	RETURN

Measurements:
	Value = mV                                   'measure voltage only
	FOR i = 1 TO 4                                    '4 measurements
		IF ms >= StartM(i) AND ms < EndM(i) THEN
			SELECT CASE i
				CASE 1 - 2
					IF ABS(Value) > ABS(Max(i)) THEN Max(i) = Value
				CASE 3
					IF Value > Max(i) THEN Max(i) = Value
				CASE ELSE
					IF Value < Max(i) THEN Max(i) = Value
			END SELECT
		END IF
		IF ms = StartM(i) AND ms = EndM(i) THEN
			Max(i) = Value
		END IF
	NEXT i
   
	RETURN

WriteMeasure:                                          'measurements to file
	OPEN "Measure.Prn" FOR APPEND AS #2
	WRITE #2, "mV", Max(1), Max(2), Max(3), Max(4)
	CLOSE #2

	RETURN

SUB Axes STATIC
	SHARED White, Red, Green, Blue, Ega
	SHARED XStart, YStart, XScale, YScale
	SHARED PlotType, LowerPlot, Time

	IF Ega THEN COLOR White
	CALL ClearLeft
	CALL ClearBottom

	XStart = 40 / XScale
	XTick = 34 / XScale
	XEnd = 540 / XScale

	YStart = 6 / YScale
	YEnd = 202 / YScale
	Y2Start = 209 / YScale
	Y2End = 272 / YScale
	Y3Start = 300 / YScale
	Y3Tick = 306 / YScale

	LINE (XStart, Y3Start)-(XEnd, Y3Start)            'time axis
	LINE (XStart, YStart)-(XStart, YEnd)              'voltage axis
	LINE (XStart, Y2Start)-(XStart, Y2End)            'current axis

	X = XStart
	Y = YStart
	Y2 = Y2Start
	FOR i = 1 TO 15                                   'draw tick marks
		LINE (XStart, Y)-(XTick, Y)
		IF i < 5 THEN LINE (XStart, Y2)-(XTick, Y2)
		IF i < 12 THEN LINE (X, Y3Start)-(X, Y3Tick)
		X = X + 50 / XScale                          '1 div = 50 time units
		Y = Y + 14 / YScale                          '1 div = 14 V units
		Y2 = Y2 + 21 / YScale                        '1 div = 21 units
	NEXT i
						   
	SELECT CASE PlotType          'Label voltage axis
		CASE 2                        'Expanded scale:
			LOCATE 1, 1                   '10 mV = 3 rows = 3 x 14 points
			PRINT "-30"
			LOCATE 4, 1
			PRINT "-40"
			LOCATE 7, 1
			PRINT "-50"
			LOCATE 10, 1
			PRINT "-60";
			LOCATE 13, 1
			PRINT "-70"
		CASE ELSE                     'Normal scale:
			LOCATE 1, 1                   '10 mV = 1 row = 14 points
			PRINT "+60"
			LOCATE 4, 1
			PRINT "+30"
			LOCATE 7, 1
			PRINT "  0"
			LOCATE 10, 1
			PRINT "-30"
			LOCATE 13, 1
			PRINT "-60"
	END SELECT

	LOCATE 23, 6                       '12.5 horiz text units = 20% max
	PRINT "0";
	LOCATE , 18
	CALL PrintNum(Time * .2)
	LOCATE , 31
	CALL PrintNum(Time * .4)
	LOCATE , 43
	CALL PrintNum(Time * .6)
	LOCATE , 56
	CALL PrintNum(Time * .8)
	LOCATE , 68
	CALL PrintNum(Time)
	PRINT " ms";

	LOCATE 16, 1
	SELECT CASE LowerPlot
		CASE 3
			PRINT "I   "                       '"I" for current
			PRINT "    "
			IF Ega THEN COLOR Green            'K current in green
			PRINT "K   "
			IF Ega THEN COLOR Blue             'Na current in blue
			PRINT "-Na "
		CASE 2
			PRINT "gate"
			IF Ega THEN COLOR Red              'h gate in red
			PRINT " h  "
			IF Ega THEN COLOR Green            'n gate in light green
			PRINT " n  "
			IF Ega THEN COLOR Blue             'm gate in light blue
			PRINT " m  "
		CASE ELSE
			PRINT "    "
			PRINT "G   "                       '"G" for conductance
			IF Ega THEN COLOR Blue
			PRINT " Na "                       'Na conductance, blue
			IF Ega THEN COLOR Green
			PRINT " K  "                       'K conductance, green
	END SELECT

	IF Ega THEN COLOR White
	LOCATE 8, 2
	PRINT "mV"                              'label voltage axis
	LOCATE 21, 1
	PRINT "STIM"                            'label stimulus indicator

END SUB

SUB GetCommand STATIC              'Check command line for simulation number
	SHARED Simulation%
    
	Option$ = COMMAND$
	Sim = VAL(Option$)
	SELECT CASE Sim
		CASE 5 TO 8                   '5 to 8 are simplified versions
			Simulation% = Sim
		CASE ELSE
	END SELECT

END SUB

SUB Parameters STATIC              'Display current parameters as a sidebar
	SHARED Simulation%
	SHARED Ega, HiRes, White, Red, Green
	SHARED SidebarCol
	SHARED Parameter()

	CALL ClearRight
	SELECT CASE Simulation%
		CASE 5
			GOSUB ActionPotentialParameters
		CASE 6
			GOSUB ExpandedScaleParameters
		CASE 7
			GOSUB DrugParameters
		CASE 8
			GOSUB IonicParameters
		CASE ELSE
			GOSUB AdvancedParameters
	END SELECT

	EXIT SUB

AdvancedParameters:                'Full sidebar for advanced option
	IF Ega THEN COLOR Green
 
	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(3, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 1 "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(3, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(3, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(3, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 2 "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(3, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(3, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(3, 7))
 
	LOCATE 11, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STX CONC.  "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "(nM)=";
	CALL PrintNum(Parameter(4, 1))
	LOCATE 13, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " TEA CONC.  "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "(mM)=";
	CALL PrintNum(Parameter(4, 2))
	LOCATE 15, SidebarCol
	IF Ega THEN COLOR Green
	PRINT "% H=";
	IF Ega THEN COLOR White
	CALL PrintNum(Parameter(4, 3))
	LOCATE 16, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " Na CONC.   "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "RATIO=";
	CALL PrintNum(Parameter(4, 4))
	LOCATE 18, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " K CONC.    "
	LOCATE , SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "RATIO=";
	CALL PrintNum(Parameter(4, 5))
 
	IF Ega THEN COLOR Red
	LOCATE 20, SidebarCol
	PRINT "            "
	LOCATE , SidebarCol
	PRINT "<F1> = MENUS"
	IF Ega THEN COLOR White

	RETURN

ActionPotentialParameters:         'Share sidebar with SimulationInstructions
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(5, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 1 "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(5, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(5, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(5, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 2 "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(5, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(5, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(5, 7))

	RETURN

ExpandedScaleParameters:
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(6, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 1 "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(6, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(6, 3))
	LOCATE 6, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(6, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS 2 "
	LOCATE 8, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(6, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(6, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(6, 7))

	RETURN

DrugParameters:
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(7, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS   "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(7, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(7, 3))
	LOCATE 6, SidebarCol
	PRINT "           ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(7, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT "PHARMACOLOGY"
	LOCATE 8, SidebarCol
	PRINT "          ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "STX=";
	CALL PrintNum(Parameter(7, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "TEA=";
	CALL PrintNum(Parameter(7, 6))
	LOCATE 10, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "%H=";
	CALL PrintNum(Parameter(7, 7))

	RETURN

IonicParameters:
	IF Ega THEN COLOR Green

	LOCATE 1, SidebarCol
	PRINT " INITIAL"
	LOCATE 2, SidebarCol
	IF Ega THEN COLOR White
	PRINT "mV=";
	CALL PrintNum(Parameter(8, 1))
	LOCATE 3, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " STIMULUS   "
	LOCATE 4, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	IF HiRes = 4 THEN
		PRINT "uA=";
	ELSE PRINT "A=";
	END IF
	CALL PrintNum(Parameter(8, 2))
	LOCATE 5, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "start=";
	CALL PrintNum(Parameter(8, 3))
	LOCATE 6, SidebarCol
	PRINT "           ";
	LOCATE , SidebarCol
	PRINT "msec=";
	CALL PrintNum(Parameter(8, 4))
	LOCATE 7, SidebarCol
	IF Ega THEN COLOR Green
	PRINT " CONC RATIOS"
	LOCATE 8, SidebarCol
	PRINT "          ";
	LOCATE , SidebarCol
	IF Ega THEN COLOR White
	PRINT "Na=";
	CALL PrintNum(Parameter(8, 5))
	LOCATE 9, SidebarCol
	PRINT "            ";
	LOCATE , SidebarCol
	PRINT "K=";
	CALL PrintNum(Parameter(8, 6))
	LOCATE 10, SidebarCol

	RETURN

END SUB

SUB PrintIntroText STATIC
	SHARED Simulation%, Called$
	SHARED Ega, White, Red, Blue, Green, Yellow
    
	IF Ega THEN COLOR Blue
	LOCATE 1, 7
	PRINT "AXOVACS Electrophysiology Computer Lab - Version 2.0"
	LOCATE , 17
	PRINT "(C) 1989 Axon Instruments, Inc."
	PRINT
    
	SELECT CASE Simulation%
		CASE 5
			GOSUB ActionPotentialText
		CASE 6
			GOSUB ExpandedScalesText
		CASE 7
			GOSUB DrugText
		CASE 8
			GOSUB IonicText
		CASE ELSE
			GOSUB AdvancedText
	END SELECT
    
	EXIT SUB

ActionPotentialText:
	PRINT
	LOCATE , 19
	IF Ega THEN COLOR Red
	PRINT "ACTION POTENTIAL SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT
	PRINT "YOU can choose the initial membrane potential,"
	PRINT "     enter stimulus parameters (two stimuli per run), and"
	PRINT "     choose to see either Na and K gates (n, m, and h),"
	PRINT "          or Na and K conductances,"
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     the membrane potential,"
	PRINT "     the gates or conductances, and"
	PRINT "     the timing of the stimuli."
	PRINT

	GOSUB PrintBottomText

	RETURN

ExpandedScalesText:
	PRINT
	LOCATE , 25
	IF Ega THEN COLOR Red
	PRINT "EXPANDED SCALES"
	IF Ega THEN COLOR White
	PRINT
	PRINT
	PRINT "YOU can choose the initial membrane potential, and"
	PRINT "     enter stimulus parameters (two stimuli per run)."
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     the membrane potential,"
	PRINT "     the Na and K currents, and"
	PRINT "     the timing of the stimuli."
	PRINT

	GOSUB PrintBottomText

	RETURN

DrugText:
	LOCATE , 26
	IF Ega THEN COLOR Red
	PRINT "PHARMACOLOGY"
	IF Ega THEN COLOR White
	PRINT
	PRINT "YOU can choose the initial membrane potential,"
	PRINT "     enter stimulus parameters (one stimulus per run),"
	PRINT "     and add simulated drugs:"
	PRINT
	PRINT "          STX (saxitoxin) to block Na channels,"
	PRINT "          TEA (tetraethylammonium) to block K channels,"
	PRINT "          or pronase to remove Na inactivation."
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     the membrane potential,"
	PRINT "     the Na and K conductances, and"
	PRINT "     the timing of the stimuli."
	PRINT

	GOSUB PrintBottomText

	RETURN

IonicText:
	LOCATE , 26
	IF Ega THEN COLOR Red
	PRINT "ION GRADIENTS"
	IF Ega THEN COLOR White
	PRINT
	PRINT "YOU can choose the initial membrane potential,"
	PRINT "     enter stimulus parameters (one stimulus per run),"
	PRINT "     and change the Na and K concentration gradients."
	PRINT ""
	PRINT "     Concentrations are entered as the ratio of"
	PRINT "          external/internal concentrations,"
	PRINT "          compared to the normal values."
	PRINT
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     the membrane potential,"
	PRINT "     the Na and K conductances, and"
	PRINT "     the timing of the stimuli."
	PRINT

	GOSUB PrintBottomText

	RETURN

AdvancedText:
	LOCATE , 19
	IF Ega THEN COLOR Red
	PRINT "ACTION POTENTIAL SIMULATION"
	IF Ega THEN COLOR White
	PRINT
	PRINT "YOU can choose the initial membrane potential,"
	PRINT "     enter stimulus parameters (two stimuli per run),"
	PRINT "     choose to see either Na and K gates (n, m, and h),"
	PRINT "          or Na and K conductances (or currents),"
	PRINT "     change the Na and/or K conductances by adding"
	PRINT "          simulated drugs (TEA, TTX, and pronase), and"
	PRINT "     change the Na and/or K concentrations."
	PRINT "THE COMPUTER will calculate and display"
	PRINT "     the membrane potential,"
	PRINT "     the gates, conductances, or currents, and"
	PRINT "     the timing of the stimuli."
	PRINT
	IF Ega THEN COLOR Green
	PRINT "The parameters that you can modify are organized in 8 menus:"
	IF Ega THEN COLOR White
	PRINT "DISPLAY:          plots, time scale, superimpose or erase"
	PRINT "STIMULI:          initial membrane potential, stimuli"
	PRINT "PHARMACOLOGY:     drugs, changes in Na and/or K concentrations"
	PRINT "OUTPUTS, MEASUREMENTS:           write data to files"
	PRINT "n, m, or h-gate KINETICS:        change model parameters"

	RETURN

PrintBottomText:
	LOCATE 19
	IF Ega THEN COLOR Yellow
	IF Called$ <> "" THEN
		PRINT "<r> to run, <e> to edit, <Esc> to go back to the main menu"
	ELSE PRINT "          <r> to run, <e> to edit, <q> to quit"
	END IF
	IF Ega THEN COLOR White

	RETURN
    
END SUB

