''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    SIMULATION OF THE ACTION POTENTIAL OF THE SQUID GIANT AXON              '
'         AFTER HODGKIN & HUXLEY, J. PHYSIOL. 117:500 (1952).                '
'                                                                            '
'    Written by Stephen W. Jones, 2/11/87; updated 2/25/89.                  '
'         Written for IBM PC-compatibles, using QuickBASIC 4.5.              '
'         Requires EGA, CGA, Hercules, or AT&T graphics adapter.             '
'                                                                            '
'    AXOVACS.BAS                                                             '
'                                                                            '
'    Program for printing introductory text, and selecting one of            '
'         the simulations available.                                         '
'                                                                            '
'    In this version, parameters are passed by writing to a 16 byte          '                                      
'         communication area in memory.                                      '
'                                                                            '
'    Include files:                                                          '
'         GRAPHICS.INC (contains subroutine Graphics and error traps)        '
'         POKEINFO.INC (writes parameters to memory)                         '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Called$ = "AXOVACS"

	True = -1
	False = NOT (True)       'Allow logical if.

	GOSUB Graphics
	GOSUB IntroText               'Print introductory text

choose:                            'Ask for choice of simulation (or quit)
	DO
	LOOP WHILE INKEY$ <> ""                 'First, flush keyboard buffer

	Row = 24
	LOCATE Row, 1
	PRINT "Enter your choice (1, 2, 3, 4, 5, 6, 7, 8, 9, or 0)";
	Column = POS(0) + 2
	INPUT ; Choice$
	Simulation% = VAL(Choice$)
	SELECT CASE Simulation%
		CASE 1
			GOSUB PokeInfo
			CLEAR
			CHAIN "CHANNEL"          'Run single channel simulation
		CASE 2 TO 4
			GOSUB PokeInfo
			CLEAR
			CHAIN "VCLAMP"           'Run voltage clamp simulation
		CASE 5 TO 9
			GOSUB PokeInfo
			CLEAR
			CHAIN "ACTION"           'Run action potential simulation
		CASE ELSE
			IF Choice$ = "0" THEN
				SCREEN 0            'If a '0' actually entered,
				CLS
				Simulation% = 0
				Called$ = ""
				GOSUB PokeInfo
				CLEAR
				END                      'exit to DOS,
			ELSE
				FOR i = Column TO 80
					LOCATE Row, i
					PRINT " ";
				NEXT i
				SOUND 260, 1
				GOTO choose              'otherwise try again.
			END IF
	END SELECT

	END

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    $INCLUDE: 'graphics.inc'

'    $INCLUDE: 'pokeinfo.inc'

IntroText:
	CLS
	LOCATE 1, 14
	IF Ega THEN COLOR Blue
	PRINT "AXOVACS Electrophysiology Computer Lab - Version 2.0"
	LOCATE , 24
	PRINT "(C) 1989 Axon Instruments, Inc."
	IF Ega THEN COLOR White
	PRINT
	PRINT "     A computer simulation, based on the model proposed by Hodgkin & Huxley"
	PRINT "[J. Physiol. 117:500 (1952)] for generation of the action potential in the"
	PRINT "squid giant axon."
	PRINT
	PRINT "    (1)  Channels            (Simulation of single channel gating)"
	PRINT
	IF Ega THEN COLOR Green
	PRINT "VOLTAGE CLAMP"
	IF Ega THEN COLOR White
	PRINT "    (2)  Conductances        (Macroscopic Na and K conductances)"
	PRINT "    (3)  Currents            (Macroscopic currents)"
	PRINT "    (4)  Advanced version    (All voltage clamp options)"
	PRINT
	IF Ega THEN COLOR Green
	PRINT "CURRENT CLAMP"
	IF Ega THEN COLOR White
	PRINT "    (5)  Action potential    (The basic simulation)"
	PRINT "    (6)  Expanded scales     (A close look at threshold)"
	PRINT "    (7)  Pharmacology        (Specific blockade of Na or K channels)"
	PRINT "    (8)  Ion substitution    (Change external Na or K concentration)"
	PRINT "    (9)  Advanced version    (All current clamp options)"
	PRINT
	PRINT "OR: (0)  Quit                (Exit to computer's operating system)"

	RETURN

