''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    EDITVAC.BAS                                                             '
'                                                                            '
'    Subprograms for VCLAMP, ACTION, and CHANNEL.BAS                         '
'                                                                            '
'    Latest revision 1/15/89 by Stephen W. Jones                             '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    $INCLUDE: 'declares.inc'

SUB CheckParam (Value$, Value, Up, Low, BadValue) STATIC
     SHARED True, False
   
     BadValue = False
     IF Value$ = "" THEN EXIT SUB
   
     NewValue = VAL(Value$)                       'Convert input to number
     IF NewValue > Up OR NewValue < Low THEN      'Is value acceptable?
          BadValue = True
     ELSE Value = NewValue                             'Update OK.
     END IF

END SUB

SUB ClearRight STATIC
     SHARED SidebarCol
   
     FOR j = 1 TO 22
          LOCATE j, SidebarCol
          PRINT "            "
     NEXT j
END SUB

SUB FindCursor STATIC
     SHARED xG, yG, XChar, YChar

     xT = POS(0)
     yT = CSRLIN
     xG = (xT - 1) * XChar
     yG = (yT - 1) * YChar

END SUB

SUB GetKey STATIC
     SHARED xG, yG, XChar, YChar, ThisKey()

     xT = POS(0)
     yT = CSRLIN
     xG = (xT - 1) * XChar
     yG = (yT - 1) * YChar

     GET (xG, yG)-(xG + XChar - 1, yG + YChar - 1), ThisKey

END SUB

SUB GetString (Text$, Tmp$) STATIC
     SHARED BackSpace$, Enter$, Down$, Up$, Cursor$, Tab$
     SHARED PgDown$, PgUp$, Home$, End$, F2$, F3$, Null$
     SHARED Esc$, RightKey$, LeftKey$, Del$, Ins$
     SHARED xG, yG, XChar, YChar, InsertMode
     SHARED True, False

'    GetString emulates (more or less) the INPUT S$ function of BASIC,
'         with special handling of <Esc> and some other special characters.
'    The previous value of the parameter to be edited is initially displayed,
'         and can be edited character-by-character.
'    The latest character entered is returned to the main
'         program, to check for special characters.

     FirstPos = POS(0)                       'Text starts here
     cLine = CSRLIN
    
     OldText$ = Tmp$                         'Tmp$:  initially, old value
     Tmp$ = ""                                    'returns last keystroke
     LongestText = 0
     GOSUB PrintOldText

DO                                      'One loop per keystroke
     TextLen = LEN(Text$)
     IF TextLen > LongestText THEN LongestText = TextLen    'used for erasing
    
     CALL FindCursor
     IF InsertMode THEN
          CALL GetKey
          CALL PutKey                   'reverse video if insert mode
     END IF
    
     DO                                 'wait 0.2 sec, unless key typed
          t = TIMER + .2
          WHILE TIMER < t
               Tmp$ = INKEY$
               IF Tmp$ <> "" THEN EXIT DO
          WEND
          IF NOT InsertMode THEN
               CALL GetUnder
               CALL PutUnder            'blinking underline in overtype mode
          END IF

          LOOP

     SELECT CASE Tmp$
          CASE Esc$                                              'lose edits;
               Text$ = OldText$
               GOSUB PrintOldText
          CASE "e", "E", F2$                                     'exit and
               IF CharNum = 1 THEN EXIT SUB
               Text$ = OldText$                                       'lose edit;
               EXIT SUB
          CASE BackSpace$                                        'del last char;
               IF CharNum > 1 THEN
                    CharNum = CharNum - 1
                    Text$ = LEFT$(Text$, CharNum - 1) + RIGHT$(Text$, TextLen - CharNum)
                    GOSUB PrintNewText
               END IF
          CASE Enter$, Down$, Up$, PgDown$, PgUp$, Home$, End$   'exit with
               IF NOT FirstKey THEN Text$ = Text$ + " "                'changes;
               EXIT SUB
          CASE "r", "R", F3$                                     'exit, run;
               IF NOT FirstKey THEN Text$ = Text$ + " "
               EXIT SUB
          CASE Tab$                                              'ignore <Tab>;
          CASE RightKey$
               CharNum = CharNum + 1                             'move right;
               IF CharNum > TextLen THEN CharNum = TextLen + 1
               GOSUB PrintNewText
          CASE LeftKey$                                          'move left;
               CharNum = CharNum - 1
               IF CharNum < 1 THEN CharNum = 1
               GOSUB PrintNewText
          CASE Del$                                              'del this char;
               IF TextLen > 0 AND TextLen >= CharNum THEN
                    Text$ = LEFT$(Text$, CharNum - 1) + RIGHT$(Text$, TextLen - CharNum)
                    IF CharNum > TextLen THEN CharNum = TextLen + 1
               END IF
               GOSUB PrintNewText
          CASE Ins$                                              'or toggle mode.
               InsertMode = NOT InsertMode
               GOSUB PrintNewText
          CASE ELSE                                         'If any other key,
               IF LEFT$(Tmp$, 1) <> Null$ THEN
               SELECT CASE InsertMode
                    CASE True                                    'insert or
                         lText$ = LEFT$(Text$, CharNum - 1)
                         rText$ = RIGHT$(Text$, TextLen - CharNum + 1)
                         Text$ = lText$ + RIGHT$(Tmp$, 1) + rText$
                    CASE ELSE                                    'overtype.
                         IF CharNum > TextLen THEN Text$ = Text$ + " "
                         MID$(Text$, CharNum) = RIGHT$(Tmp$, 1)
                         IF FirstKey THEN Text$ = RIGHT$(Tmp$, 1)
               END SELECT
               CharNum = CharNum + 1
               FirstKey = False
               GOSUB PrintNewText
               END IF
     END SELECT
     
     Tmp$ = ""
    
     LOOP                                         'Get another key

PrintOldText:                                'Start over.
     CharNum = 1                                  '(re)initialize
     Text$ = OldText$
     FOR i = 1 TO LongestText + 1
          LOCATE cLine, FirstPos + i - 1          'clear any old values
          PRINT " "
     NEXT i
     FirstKey = True                              'allow quick exit next
    
     CALL LocateCursor(cLine, FirstPos)
     PRINT OldText$                               'print default
     CALL LocateCursor(cLine, FirstPos)

     RETURN

PrintNewText:                                'Display new value.
     FOR i = 1 TO LongestText + 1
          LOCATE cLine, FirstPos + i - 1          'clear last value
          PRINT " "
     NEXT i
     CALL LocateCursor(cLine, FirstPos)
     PRINT Text$;                                 'print value
     CALL LocateCursor(cLine, FirstPos + CharNum - 1)
     FirstKey = False

     RETURN

END SUB

SUB GetUnder STATIC
     SHARED xG, yG, XChar, YChar, UnderLine()
 
     yC = yG + YChar - 1
     GET (xG, yC)-(xG + XChar - 1, yC), UnderLine

END SUB

SUB LocateCursor (yT, xT) STATIC
     SHARED xG, yG, XChar, YChar

     LOCATE yT, xT
     xG = (xT - 1) * XChar
     yG = (yT - 1) * YChar

END SUB

SUB MenuInstructions STATIC
     SHARED Ega, White, Green
     SHARED SidebarCol
     SHARED Called$
   
     CALL ClearRight

     IF Ega THEN COLOR Green                     'Print list of menus.
     LOCATE 1, SidebarCol
     PRINT "   MENUS    "
     IF Ega THEN COLOR White
     LOCATE , SidebarCol
     PRINT "<d>isplay   "
     LOCATE , SidebarCol
     PRINT "<s>timuli   "
     LOCATE , SidebarCol
     PRINT "<p>harmacol."
     LOCATE , SidebarCol
     PRINT "<o>utputs   "
     LOCATE , SidebarCol
     PRINT "me<a>sure   "
     LOCATE , SidebarCol
     PRINT "<n> gate or "
     LOCATE , SidebarCol
     PRINT "<m> gate or "
     LOCATE , SidebarCol
     PRINT "<h> gate    "
     LOCATE , SidebarCol
     PRINT "   kinetics "
     PRINT
     IF Ega THEN COLOR Green
     LOCATE , SidebarCol
     IF Called$ = "" THEN
          PRINT "   OTHER    "                    'save a line if needed
          LOCATE , SidebarCol
     END IF
     PRINT "  OPTIONS   "
     IF Ega THEN COLOR White
     LOCATE , SidebarCol
     PRINT "<r>un       "
     LOCATE , SidebarCol
     PRINT "<v>alues    "
     LOCATE , SidebarCol
     PRINT "<e>dit      "
     LOCATE , SidebarCol
     PRINT "<i>nitialize"
     LOCATE , SidebarCol
     PRINT "   values to"
     LOCATE , SidebarCol
     PRINT "   defaults "
     LOCATE , SidebarCol
     PRINT "<c>lear the "
     LOCATE , SidebarCol
     PRINT "   screen   "
     LOCATE , SidebarCol
     IF Called$ = "" THEN
          PRINT "<q>uit      "
     ELSE
          PRINT "<Esc> for   "
          LOCATE , SidebarCol
          PRINT "   main menu"
     END IF

END SUB

SUB NoSpace (Number, Number$) STATIC    'Convert to text; remove leading blanks
     Number$ = STR$(Number)

     Length = LEN(Number$)
     WHILE LEFT$(Number$, 1) = " "
          Number$ = RIGHT$(Number$, Length - 1)
     WEND

END SUB

SUB PeekInfo STATIC      'Get name of calling program and desired simulation
     SHARED Simulation%, Called$

     PeekText$ = ""
     DEF SEG = &H4F           'Start of a 16-byte "communications area"
     FOR i = 1 TO 16
          PeekVal = PEEK(i - 1)                   'Read byte
          PeekChar$ = LEFT$(MKI$(PeekVal), 1)     'change to text
          PeekText$ = PeekText$ + PeekChar$
     NEXT i

     SafetyCheck$ = LEFT$(PeekText$, 7)
     IF SafetyCheck$ = "axovacs" THEN             'written by axovacs?
          Called$ = RIGHT$(PeekText$, 8)
          WHILE RIGHT$(Called$, 1) = " "
               Called$ = LEFT$(Called$, LEN(Called$) - 1)   'program name
          WEND
          Simulation% = VAL(MID$(PeekText$, 8, 1))          'simulation#

          FOR i = 0 TO 15                         'clear communications area
               Char$ = " "
               PokeData = ASC(Char$)
               POKE i, PokeData
          NEXT i
     ELSE
          Called$ = ""                            'defaults
          Simulation% = 0
     END IF
    
     i = 1
     DO WHILE LEN(Called$) >= i                   'crude check of file name
          CharVal = ASC(MID$(Called$, i, 1))
          IF CharVal < 33 OR CharVal > 126 THEN Called$ = ""
          i = i + 1
     LOOP

     DEF SEG

END SUB

SUB PrintDefaultOption (Value, StartLine, Options) STATIC
     SHARED PrintDefaultAt
   
     LOCATE StartLine - Options + Value - 1, PrintDefaultAt
     PRINT "Yes "
     LOCATE StartLine

END SUB

SUB PrintDefaultValue (Value, Row) STATIC
     SHARED PrintDefaultAt
   
     LOCATE Row, PrintDefaultAt              'Print on same line
     CALL PrintNum(Value)                    'Print w/o leading blanks
   
     PRINT                                   'Move to next line

END SUB

SUB PrintNum (Number) STATIC       'Routine to print number w/o leading blank
     Number$ = STR$(Number)
StripSpace:
          Length = LEN(Number$)
          IF LEFT$(Number$, 1) = " " THEN
               Number$ = RIGHT$(Number$, Length - 1)
               GOTO StripSpace
          END IF
      PRINT Number$;                         'Stay on same line.

END SUB

SUB PutKey STATIC
     SHARED xG, yG, XChar, YChar, ThisKey()

     xT = POS(0)
     yT = CSRLIN
     xG = (xT - 1) * XChar
     yG = (yT - 1) * YChar
     
     PUT (xG, yG), ThisKey, PRESET           'reverse video

END SUB

SUB PutUnder STATIC
     SHARED xG, yG, XChar, YChar, UnderLine()
 
     yC = yG + YChar - 1
     PUT (xG, yC), UnderLine, PRESET

END SUB

SUB ShowBigCursor STATIC
     SHARED xG, yG, XChar, YChar, BigCursor()

     xT = POS(0)
     yT = CSRLIN
     xG = (xT - 1) * XChar
     yG = (yT - 1) * YChar

     GET (xG, yG)-(xG + 4 * XChar - 1, yG + YChar - 1), BigCursor
     PUT (xG, yG), BigCursor, PRESET

END SUB

