''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    FILEOUT.INC                                                             '
'                                                                            '
'    Some subroutines for initializing output files, for ACTION and VCLAMP   '
'         GetFileName is used both for Lotus- and pClamp-style output files; '
'              the others are used only for pClamp-style files.              '
'                                                                            '
'    Revised 2/11/89 by Stephen W. Jones                                     '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dimensions:                             'Dimension variables
     DIM ClampexParam(80)
    
     TYPE Header                        'used for writing CLAMPEX-style files
          Par AS STRING * 320
          Comm AS STRING * 77
          Label1 AS STRING * 16
          Label2 AS STRING * 16
          Label3 AS STRING * 16
          Label4 AS STRING * 16
          Label5 AS STRING * 16
          Env AS STRING * 35
     END TYPE
     DIM ClampexHeader AS Header
     DIM ClampexParameter AS STRING * 4
     DIM ClampexData AS INTEGER

     RETURN

GetFileName:
     d$ = DATE$                                   'pCLAMP-style file name
     Month$ = MID$(d$, 2, 1)
     IF LEFT$(d$, 1) = "1" THEN
          IF Month$ = "0" THEN Month$ = "o"
          IF Month$ = "1" THEN Month$ = "n"
          IF Month$ = "2" THEN Month$ = "d"
     END IF
     Year$ = RIGHT$(d$, 2)
     Day$ = MID$(d$, 4, 2)
     Prefix$ = Year$ + Month$ + Day$ + "c"
  
     SELECT CASE FileOut
          CASE 2
               MID$(Prefix$, 6, 1) = SpecialChar$     'after # for Lotus
          CASE 3
               MID$(Prefix$, 1, 1) = SpecialChar$     'first char. for pCLAMP
          CASE ELSE
     END SELECT
 
     FileNumber = FileNumber + 1
     CALL NoSpace(FileNumber, FileNum$)
  
     SELECT CASE LEN(FileNum$)                         'Adjust name length
          CASE 1
               FileNum$ = "0" + FileNum$
          CASE IS > 2
               FileNum$ = RIGHT$(FileNum$, 2)
     END SELECT
 
     FileName$ = Prefix$ + FileNum$ + FileExt$
 
     RETURN

MakeClampexParameters:                  'Header info for CLAMPEX-type file.
     FOR i = 1 TO 80
          ClampexParam(i) = 0                'Set most to zero
     NEXT i

     SELECT CASE Tint
          CASE IS < 1000
               ClampexParam(24) = 1          '# of 512 point segments
          CASE 1000 TO 1499
               ClampexParam(24) = 2
          CASE 1500 TO 1999
               ClampexParam(24) = 3
          CASE IS >= 2000                    'not all points may be written
               ClampexParam(24) = 4
     END SELECT
     FirstPoints = 12 * ClampexParam(24)
     PointsToWrite = 500 * ClampexParam(24)

     ClampexParam(1) = 1
     ClampexParam(2) = 1                          'Number of channels?
     ClampexParam(3) = FirstPoints + PointsToWrite
     ClampexParam(4) = 1                          'Number of episodes?
     ClampexParam(5) = dt * 1000
     ClampexParam(9) = 4                          'Version number
     ClampexParam(12) = 1                         '# runs/trial
     ClampexParam(13) = 1                         '# episodes
     ClampexParam(17) = 1                         'starting episode
     ClampexParam(14) = dt * 1000                 'digitizing interval
     ClampexParam(18) = 4                         'command mode = simple
     ClampexParam(26) = HP                        'A level
     ClampexParam(28) = VStim1                    'B level
     ClampexParam(33) = .02                       'fake DAC gain
     ClampexParam(44) = VStim2                    'C level
     ClampexParam(49) = OutputGain                'mV output / mV cell
     ClampexParam(61) = 1                         'display factor ch0
     ClampexParam(63) = 1                         'display factor ch1
     ClampexParam(53) = 10                        'ADC volts full scale
     ClampexParam(54) = 10                        'DAC voltage range
     ClampexParam(55) = 12                        'number of bits ADC
     ClampexParam(56) = 12                        'number of bits DAC

     OPEN FileName$ FOR RANDOM AS #1 LEN = 4      'Open file, 4 byte records
     GET #1, 1                                    'Try reading file
     IF NOT EOF(1) THEN                                     'if file has data,
          CLOSE #1
          OldFile$ = Prefix$ + FileNum$ + ".bak"
          OPEN OldFile$ FOR RANDOM AS #1 LEN = 4
          CLOSE #1
          KILL OldFile$                                'delete any old .bak,
          NAME FileName$ AS OldFile$                   'save old .dat as .bak,
          OPEN FileName$ FOR RANDOM AS #1 LEN = 4           'open new file.
     END IF
     FOR i = 1 TO 80
          ClampexParameter = MKSMBF$(ClampexParam(i))
          PUT #1, i, ClampexParameter                  'Write to file header
     NEXT i
     CLOSE #1                                     'Close file
 
     RETURN

MakeClampexLabels:
     OPEN FileName$ FOR RANDOM AS #1 LEN = LEN(ClampexHeader)    '512 bytes
     GET #1, 1, ClampexHeader                                    'Save param.
    
     ClampexHeader.Comm = Comment$                               'Comment
     ClampexHeader.Label1 = "                "                   'Axis labels
     ClampexHeader.Label2 = "              mV"
     ClampexHeader.Label3 = "                "
     ClampexHeader.Label4 = Label$
     ClampexHeader.Label5 = "              mV"
     ClampexHeader.Env = ""                                      '?

     PUT #1, 1, ClampexHeader                               'Rewrite header
     CLOSE #1                                               'Close file

     RETURN

