''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    GRAPHICS.INC                                                            '
'                                                                            '
'    Routine to determine graphics mode, used for most AXOVACS modules.      '
'         Includes error traps, at main program level.                       '
'                                                                            '
'    Revised 1/15/89 by Stephen W. Jones                                     '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
EGAerror:
     ErrorType = ERR                         'Error traps for Graphics
     IF ErrorType = 5 THEN RESUME tryHerc         'illegal function call
     ON ERROR GOTO 0

HERCerror:
     ErrorType = ERR
     IF ErrorType = 5 THEN RESUME tryATT
     ON ERROR GOTO 0

ATTerror:
     ErrorType = ERR
     IF ErrorType = 5 THEN RESUME tryCga
     ON ERROR GOTO 0

CGAerror:
     ErrorType = ERR
     IF ErrorType = 5 THEN RESUME needGraphics
     ON ERROR GOTO 0

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Graphics:
     Black = 0
     Ega = True                    'Assume EGA color monitor to start.
     Mono = False
     HiRes = 9
     YScale = 1
     XScale = 1

     DEF SEG = 0
     VideoMode = PEEK(&H449)
     IF VideoMode = 7 OR VideoMode = 15 THEN      'Test for EGA monochrome
          HiRes = 10
          Mono = True
     END IF

     CLS                                'clear entire screen
     ON ERROR GOTO EGAerror             'assume error means non-EGA monitor
     SCREEN HiRes                       '640 x 350 resolution
     WIDTH 80, 25                       'Force to 25 line mode
     LINE (1, 1)-(1, 1)                 'crude test to see if graphics works.
     CLS

     WINDOW
     VIEW                               'clear leftover viewports

     IF Mono THEN                       'use these instead of colors
          Green = 1                          '1 = low intensity white
          Blue = 3                           '3 = bright white
          Red = 3
          Yellow = 3
          White = 3
     ELSE
          Green = 10                    'use these as names for colors
          Blue = 11
          Red = 12
          Yellow = 13
          White = 15
          
          DIM PAL%(16)                  'change default colors
          FOR i = 0 TO 15
               PAL%(i) = 0              'colors not used set to black
          NEXT i
          PAL%(White) = 63              'foreground still bright white
          PAL%(Green) = 18              'a better green
          PAL%(Blue) = 11               'a better blue
          PAL%(Red) = 36                'a better red
          PAL%(Yellow) = 54             'yellow
          PALETTE USING PAL%(0)
     END IF

     GOTO trapOff

tryHerc:
     ON ERROR GOTO HERCerror

     Ega = False              'If EGA doesn't work, try Hercules next.
     Mono = True
     HiRes = 3
     YScale = 1
     XScale = 640 / 720            'Adjust horizontal scaling
    
     CLS
     SCREEN HiRes
     LINE (1, 1)-(1, 1)            'A crude test to see if graphics works
     CLS
     GOTO trapOff

tryATT:
     ON ERROR GOTO ATTerror
    
     Ega = False              'Next, try the AT&T 640 x 480 monochrome mode
     Mono = True
     HiRes = 4
     YScale = 350 / 400            'Adjust vertical scaling
     XScale = 1

     CLS
     SCREEN HiRes
     COLOR 15
     LINE (1, 1)-(1, 1)            'test graphics
     CLS
     GOTO trapOff

tryCga:
     ON ERROR GOTO CGAerror

     Ega = False              'Finally, try CGA 640 x 200 monochrome mode
     Mono = True
     HiRes = 2
     YScale = 350 / 200            'Adjust vertical scaling
     XScale = 1

     CLS
     SCREEN HiRes
     LINE (1, 1)-(1, 1)            'test graphics
     CLS
     GOTO trapOff

needGraphics:
     CLS
     PRINT "This program requires a graphics adapter."
     PRINT "     To run AXOVACS with a Hercules card,"
     PRINT "     run MSHERC first."
         
     END

trapOff:
     ON ERROR GOTO 0

     XChar = 8 / XScale       'to convert graphics units to text units
     YChar = 14 / YScale

     RETURN

