''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'                                                                            '
'    PARAM.INC                                                               '
'                                                                            '
'    Subroutines common to SUB NewParameters in ACTION and VCLAMP            '
'                                                                            '
'    Revised 1/15/89 by Stephen W. Jones                                     '
'                                                                            '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'    Menu numbering:
'         1 = [not used]
'         2 = Display
'         3 = Stimuli
'         4 = Pharamcology
'         {simplified menus; number varies}
'         OutputMenuN = Outputs
'         {other advanced menus, if any}
'         LastMenuN

Dimensions:
     ArraySize = 16300                  'Large storage space reserved for menu
     DIM MenuSpace&(ArraySize)          'This stores data while menu is active

     DIM InputLine(20, 20)
     DIM CursorStart(20, 20)
     DIM ParameterType(20, 20)
     DIM UpperL(20, 20)
     DIM LowerL(20, 20)
     DIM ParameterToErase(20, 20)
     DIM LinesToErase(20, 20)
     DIM OptionCol(20)
     DIM NumberOfParameters(20)

     None = 0                           'Menu status indicators
     Big = 1
     Small = 2

     OptionType = 1                     'input variable types
     ValueType = 2
     MultipleType = 3
     TextType = 4
   
     MaxTimeScale = 10                  '10 timescale options

     OptionCol(0) = PrintDefaultAt      'for options within menu
     TabCol = 5                         'for tabs within menu
     FOR i = 1 TO 8
          OptionCol(i) = OptionCol(i - 1) + TabCol
     NEXT i
     Blanks$ = ""
     
     FOR i = 1 TO 9
          Blanks$ = Blanks$ + " "       'used for clearing old values
     NEXT i

     DoneAlready = True                 'do this first time through only
     InsertMode = False                 'Overtype mode for editing

     RETURN

DefineKeys:
     Null$ = CHR$(0)                    'used to read these from keyboard
     Esc$ = CHR$(27)
     Enter$ = CHR$(13)
     SpaceBar$ = CHR$(32)
     BackSpace$ = CHR$(8)
     RightKey$ = Null$ + CHR$(77)
     LeftKey$ = Null$ + CHR$(75)
     Down$ = Null$ + CHR$(80)
     Up$ = Null$ + CHR$(72)
     PgDown$ = Null$ + CHR$(81)
     PgUp$ = Null$ + CHR$(73)
     Home$ = Null$ + CHR$(71)
     End$ = Null$ + CHR$(79)
     F1$ = Null$ + CHR$(59)
     F2$ = Null$ + CHR$(60)
     F3$ = Null$ + CHR$(61)
     Tab$ = CHR$(9)
     Ins$ = Null$ + CHR$(82)
     Del$ = Null$ + CHR$(83)

     UpArrow$ = CHR$(24)                'used to print these characters
     DownArrow$ = CHR$(25)
     RightArrow$ = CHR$(26)
     LeftArrow$ = CHR$(27)
     Check$ = CHR$(251)
     Cursor$ = CHR$(219)
     IF HiRes = 4 THEN Cursor$ = BackSpace$

     RETURN

QuitMenu:                               'Leave program?
     MenuTitle$ = ""
     GOSUB GotKey
     LOCATE 10, 15
     IF Ega THEN COLOR Yellow
     IF Called$ <> "" THEN
          PRINT "Enter <y> to return to the main menu. " + Cursor$;
     ELSE
          PRINT " Do you really want to quit (y/n)? " + Cursor$;
     END IF
     ReallyQuit$ = ""
     DO WHILE LEN(ReallyQuit$) = 0
          ReallyQuit$ = INKEY$
     LOOP
     IF LCASE$(ReallyQuit$) = "y" THEN            'if confirmed,
          SCREEN 0
          CLS
          IF Called$ <> "" THEN
               CHAIN Called$                           'return to menu, or
          ELSE
               END                                     'exit to DOS.
          END IF
     ELSE
          IF Ega THEN COLOR White
          GOSUB DoneWithMenu                      'otherwise, get another input.
     END IF

     RETURN
      
GotKey:
     IF MenuOnScreen = None THEN   'Large or small menu box?
          NewMenuSize = True
     ELSE NewMenuSize = False
     END IF
  
     IF MenuTitle$ = "" THEN       'Small menu box for QuitMenu
          x1 = 9
          x2 = 58                            'text coordinate units
          y1 = 8
          y2 = 13
          IF MenuOnScreen = Big THEN NewMenuSize = True
          MenuOnScreen = Small
     ELSE                          'Other menus have large boxes
          x1 = 11                   'This includes most of data plotting area
          x2 = 61
          y1 = 1
          y2 = 20
          IF MenuOnScreen = Small THEN NewMenuSize = True
          MenuOnScreen = Big
     END IF

     IF NewMenuSize THEN                'Save current screen information
          xL = CINT((x1 - 1) * xChar)
          xH = CINT((x2 - 1) * xChar)        'graphics units
          yL = CINT((y1 - 1) * yChar)
          yH = CINT((y2 - 1) * yChar)

          GET (xL, yL)-(xH, yH), MenuSpace&       'Save in array MenuSpace&
     END IF
     
     VIEW (xL, yL)-(xH, yH)                       'clear viewport
     CLS
     VIEW

     IF Ega THEN COLOR Green
     x1a = xL + xChar / 2
     x2a = xH - xChar / 2
     y1a = yL + yChar / 2
     y2a = yH - yChar / 2
     LINE (x1a, y1a)-(x2a, y2a), , B              'Plot box inside viewport

     IF Ega THEN COLOR Blue                       'Standard menu header
     LOCATE 2, OptionCol(0)
     PRINT MenuTitle$;
     LOCATE , PrintDefaultAt

     IF Ega THEN COLOR White

     RETURN

DoneWithMenu:
     PUT (xL, yL), MenuSpace&, PSET               'Restore screen
     IF NOT Simple AND NOT RunNow AND MenuTitle$ <> "" THEN
          CALL MenuInstructions
     END IF

     RETURN

GetInputs:                         'Parameter input routine for ALL menus.
anotherInput:
     SELECT CASE ParameterType(MenuN, ParamN)     'So far, 3 variable types
          CASE OptionType
               GOTO optionInput
          CASE ValueType
               GOTO valueInput
          CASE MultipleType
               GOTO multipleOptionInput
     END SELECT
     GOTO anotherInput

optionInput:
     LineY = InputLine(MenuN, ParamN)
     CALL LocateCursor(LineY, CursorStart(MenuN, ParamN))
     CALL ShowBigCursor                                'Toggle normal-inverted
 
     InputKey$ = ""
     DO WHILE LEN(InputKey$) = 0                       'Get a key
          InputKey$ = INKEY$
     LOOP
  
     SELECT CASE InputKey$
          CASE SpaceBar$, "y", "n", "Y", "N"                'New value
               CALL ShowBigCursor
               GOSUB ChangeOption
          CASE Down$, Enter$                                'Move down
               CALL ShowBigCursor
               ParamN = ParamN + 1
               IF ParamN > NumberOfParameters(MenuN) THEN
                    ParamN = 1
               END IF
          CASE Up$                                          'Move up
               CALL ShowBigCursor
               ParamN = ParamN - 1
               IF ParamN < 1 THEN
                    ParamN = NumberOfParameters(MenuN)
               END IF
          CASE Home$                                        'Top of screen
               CALL ShowBigCursor
               ParamN = 1
          CASE End$                                         'Bottom of screen
               CALL ShowBigCursor
               ParamN = NumberOfParameters(MenuN)
          CASE PgDown$                                      'Switch menus
               NextMenu = MenuN + 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 5
                         NextMenu = OutputMenuN
                    CASE IS > LastMenuN
                         NextMenu = 2
               END SELECT
               RETURN
          CASE PgUp$
               NextMenu = MenuN - 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 1
                         NextMenu = LastMenuN
                    CASE OutputMenuN - 1
                         NextMenu = 4
               END SELECT
               RETURN
          CASE "r", "R", F3$                                'Do a run
               RunNow = True
               RETURN
          CASE "e", "E", F2$                                'Leave menus
               RETURN
          CASE ELSE                                         'get another key
               CALL ShowBigCursor
     END SELECT
 
     GOTO anotherInput             '<Esc>, <Enter>, <Pg Up>, <Pg Dn> exit loop

multipleOptionInput:
     LineY = InputLine(MenuN, ParamN)
     CALL LocateCursor(LineY, CursorStart(MenuN, ParamN))
     CALL ShowBigCursor                                'Toggle normal-inverted
     PN = ParamN
     IF MenuN = 2 THEN PN = ParamN - ExtraValues  'Display menu numbering odd
 
     InputKey$ = ""
     DO WHILE LEN(InputKey$) = 0                       'Get a key
          InputKey$ = INKEY$
     LOOP
  
     SELECT CASE InputKey$
          CASE "y", "Y", RightKey$, SpaceBar$               'Increase value
               GOSUB IncrementOption
          CASE "n", "N", LeftKey$                           'Decrease value
               GOSUB DecrementOption
          CASE Down$, Enter$                                'Move down
               CALL ShowBigCursor
               ParamN = ParamN + 1
               IF ParamN > NumberOfParameters(MenuN) THEN
                    ParamN = 1
               END IF
          CASE Up$                                          'Move up
               CALL ShowBigCursor
               ParamN = ParamN - 1
               IF ParamN < 1 THEN
                    ParamN = NumberOfParameters(MenuN)
               END IF
          CASE Home$                                        'Top of screen
               CALL ShowBigCursor
               ParamN = 1
          CASE End$                                         'Bottom of screen
               CALL ShowBigCursor
               ParamN = NumberOfParameters(MenuN)
          CASE PgDown$                                      'Switch menus
               NextMenu = MenuN + 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 5
                         NextMenu = OutputMenuN
                    CASE IS > LastMenuN
                         NextMenu = 2
               END SELECT
               RETURN
          CASE PgUp$
               NextMenu = MenuN - 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 1
                         NextMenu = LastMenuN
                    CASE OutputMenuN - 1
                         NextMenu = 4
               END SELECT
               RETURN
          CASE "r", "R", F3$                                'Do a run
               RunNow = True
               RETURN
          CASE "e", "E", F2$                                'Leave menus
               RETURN
          CASE ELSE                                         'get another key
               CALL ShowBigCursor
     END SELECT
 
     GOTO anotherInput             '<Esc>, <Enter>, <Pg Up>, <Pg Dn> exit loop

valueInput:
     CALL NoSpace(Parameter(MenuN, ParamN), OldValue$)
     CALL LocateCursor(InputLine(MenuN, ParamN), OptionCol(0))
     CALL GetString(Text$, OldValue$)        'Get the new value
    
     InputKey$ = OldValue$                   'special key may be returned
     CALL CheckParam(Text$, Parameter(MenuN, ParamN), UpperL(MenuN, ParamN), LowerL(MenuN, ParamN), BadInput)
    
     LOCATE , OptionCol(0)
     PRINT "         ";                      'Clear data entry area, and
     LOCATE , OptionCol(0)
     CALL PrintNum(Parameter(MenuN, ParamN))      'display current value.
    
     IF BadInput THEN                   'If the value entered was out of range,
          SOUND 200, 1                       'sound a warning tone,
          GOTO anotherInput                  'and don't move cursor.
     END IF

     SELECT CASE InputKey$                             'What to do next?
          CASE "r", "R", F3$                                'Do a run
               RunNow = True
               RETURN
          CASE "e", "E", F2$                                'Leave menus
               RETURN
          CASE Down$, Enter$                                'Move down
               ParamN = ParamN + 1
               IF ParamN > NumberOfParameters(MenuN) THEN
                    ParamN = 1
               END IF
          CASE Up$                                          'Move up
               ParamN = ParamN - 1
               IF ParamN < 1 THEN
                    ParamN = NumberOfParameters(MenuN)
               END IF
          CASE Home$                                        'Top of screen
               ParamN = 1
          CASE End$                                         'Bottom of screen
               ParamN = NumberOfParameters(MenuN)
          CASE PgDown$                                      'Switch menus
               NextMenu = MenuN + 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 5
                         NextMenu = OutputMenuN
                    CASE IS > LastMenuN
                         NextMenu = 2
               END SELECT
               RETURN
          CASE PgUp$
               NextMenu = MenuN - 1
               SELECT CASE NextMenu                    'avoid "simple" menus
                    CASE 1
                         NextMenu = LastMenuN
                    CASE OutputMenuN - 1
                         NextMenu = 4
               END SELECT
               RETURN
          CASE ELSE
     END SELECT

     GOTO anotherInput             '<Enter>, <Esc>, <Pg Up>, <Pg Dn> exit loop
 
     RETURN

ChangeOption:                      'Procedure varies for each parameter
     SELECT CASE MenuN
          CASE 2
               GOSUB ChangeDisplayOption     'Display menu
          CASE OutputMenuN
               GOSUB ChangeOutputOption      'Output menu
          CASE ELSE
               GOSUB SwitchTrueFalse         'simplified menus, kinetics menus
     END SELECT

     RETURN

ChangeOutputOption:                                    'FileOut
     SELECT CASE LCASE$(InputKey$)
          CASE "n"
               IF ParamN <> Parameter(MenuN, 1) THEN RETURN
               SELECT CASE ParamN
                    CASE 1
                         Parameter(MenuN, 1) = 2
                         CursorLine = CSRLIN + 1
                         ParamN = 2
                    CASE 2
                         Parameter(MenuN, 1) = 3
                         CursorLine = CSRLIN + 1
                         ParamN = 3
                    CASE ELSE
                         Parameter(MenuN, 1) = 1
                         CursorLine = CSRLIN - 2
                         ParamN = 1
               END SELECT
          CASE ELSE
               CursorLine = CSRLIN
               SELECT CASE ParamN
                    CASE 1
                         Parameter(MenuN, 1) = 1
                    CASE 2
                         Parameter(MenuN, 1) = 2
                    CASE ELSE
                         Parameter(MenuN, 1) = 3
               END SELECT
     END SELECT
     GOSUB RewriteDefault

     RETURN

SwitchTrueFalse:
     PN = ParamN
     IF MenuN = 2 THEN PN = ParamN - ExtraValues  'Display menu numbering odd
     LOCATE InputLine(MenuN, ParamN), PrintDefaultAt
     SELECT CASE InputKey$
          CASE SpaceBar$                                    'toggle
               Parameter(MenuN, PN) = NOT Parameter(MenuN, PN)
               SELECT CASE Parameter(MenuN, PN)
                    CASE True
                         PRINT "Yes "
                    CASE ELSE
                         PRINT "No  "
               END SELECT
          CASE "y", "Y"                                     'Yes
               Parameter(MenuN, PN) = True
               PRINT "Yes "
          CASE "n", "N"                                     'No
               Parameter(MenuN, PN) = False
               PRINT "No  "
          CASE ELSE
     END SELECT

     RETURN

RewriteDefault:
     StartErase = InputLine(MenuN, ParameterToErase(MenuN, ParamN))
     EndErase = LinesToErase(MenuN, ParamN) + StartErase - 1

     FOR i = StartErase TO EndErase          'Clears value from menu
          LOCATE i, PrintDefaultAt
          PRINT Blanks$
     NEXT i

     LOCATE CursorLine, PrintDefaultAt       'Mark currently active option
     PRINT "Yes "

     RETURN

IncrementOption:
     Parameter(MenuN, PN) = Parameter(MenuN, PN) + 1
     IF Parameter(MenuN, PN) > .5 + MaxTimeScale THEN Parameter(MenuN, PN) = 1
     GOSUB RewriteTimeOption

     RETURN

DecrementOption:
     Parameter(MenuN, PN) = Parameter(MenuN, PN) - 1
     IF Parameter(MenuN, PN) < .5 THEN Parameter(MenuN, PN) = MaxTimeScale
     GOSUB RewriteTimeOption

     RETURN

RewriteTimeOption:
     LOCATE , PrintDefaultAt
     PRINT Blanks$;
     LOCATE , PrintDefaultAt
     CALL PrintNum(TimeOption(Parameter(MenuN, PN)))

     RETURN

